@@ -5,7 +5,7 @@
  * Copyright (C) 2003  CodeFactory AB
  *
  * Licensed under the Academic Free License version 2.1
- * 
+ *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
@@ -15,7 +15,7 @@
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
@@ -36,6 +36,7 @@
 #include "policy.h"
 #include "bus.h"
 #include "selinux.h"
+#include "apparmor.h"
 
 struct BusService
 {
@@ -75,6 +76,7 @@ bus_registry_new (BusContext *context)
 {
   BusRegistry *registry;
 
+  _dbus_assert (context);
   registry = dbus_new0 (BusRegistry, 1);
   if (registry == NULL)
     return NULL;
@@ -368,7 +370,7 @@ bus_registry_list_services (BusRegistry *registry,
   
  error:
   for (j = 0; j < i; j++)
-    dbus_free (retval[i]);
+    dbus_free (retval[j]);
   dbus_free (retval);
 
   return FALSE;
@@ -385,13 +387,13 @@ bus_registry_acquire_service (BusRegistry      *registry,
 {
   dbus_bool_t retval;
   DBusConnection *old_owner_conn;
-  DBusConnection *current_owner_conn;
   BusClientPolicy *policy;
   BusService *service;
   BusActivation  *activation;
   BusSELinuxID *sid;
   BusOwner *primary_owner;
- 
+  int limit;
+
   retval = FALSE;
 
   if (!_dbus_validate_bus_name (service_name, 0,
@@ -459,9 +461,13 @@ bus_registry_acquire_service (BusRegistry      *registry,
                       _dbus_string_get_const_data (service_name));
       goto out;
     }
+
+  if (!bus_apparmor_allows_acquire_service (connection,
+                                            bus_context_get_type (registry->context),
+                                            _dbus_string_get_const_data (service_name), error))
+    goto out;
   
-  if (!bus_client_policy_check_can_own (policy, connection,
-                                        service_name))
+  if (!bus_client_policy_check_can_own (policy, service_name))
     {
       dbus_set_error (error, DBUS_ERROR_ACCESS_DENIED,
                       "Connection \"%s\" is not allowed to own the service \"%s\" due "
@@ -473,15 +479,24 @@ bus_registry_acquire_service (BusRegistry      *registry,
       goto out;
     }
 
-  if (bus_connection_get_n_services_owned (connection) >=
-      bus_context_get_max_services_per_connection (registry->context))
+  limit = bus_context_get_max_services_per_connection (registry->context);
+
+  if (bus_connection_get_n_services_owned (connection) >= limit)
     {
-      dbus_set_error (error, DBUS_ERROR_LIMITS_EXCEEDED,
+      DBusError tmp_error;
+
+      dbus_error_init (&tmp_error);
+      dbus_set_error (&tmp_error, DBUS_ERROR_LIMITS_EXCEEDED,
                       "Connection \"%s\" is not allowed to own more services "
-                      "(increase limits in configuration file if required)",
+                      "(increase limits in configuration file if required; "
+                      "max_names_per_connection=%d)",
                       bus_connection_is_active (connection) ?
                       bus_connection_get_name (connection) :
-                      "(inactive)");
+                      "(inactive)",
+                      limit);
+      bus_context_log (registry->context, DBUS_SYSTEM_LOG_WARNING,
+          "%s", tmp_error.message);
+      dbus_move_error (&tmp_error, error);
       goto out;
     }
   
@@ -510,12 +525,10 @@ bus_registry_acquire_service (BusRegistry      *registry,
   primary_owner = bus_service_get_primary_owner (service);
   if (primary_owner == NULL)
     goto out;
-    
-  current_owner_conn = primary_owner->conn;
-     
+
   if (old_owner_conn == NULL)
     {
-      _dbus_assert (current_owner_conn == connection);
+      _dbus_assert (primary_owner->conn == connection);
 
       *result = DBUS_REQUEST_NAME_REPLY_PRIMARY_OWNER;      
     }
@@ -592,8 +605,9 @@ bus_registry_acquire_service (BusRegistry      *registry,
   activation = bus_context_get_activation (registry->context);
   retval = bus_activation_send_pending_auto_activation_messages (activation,
 								 service,
-								 transaction,
-								 error);
+								 transaction);
+  if (!retval)
+    BUS_SET_OOM (error);
   
  out:
   return retval;
@@ -656,7 +670,7 @@ bus_registry_release_service (BusRegistry      *registry,
     {
       *result = DBUS_RELEASE_NAME_REPLY_NON_EXISTENT;
     }
-  else if (!bus_service_has_owner (service, connection))
+  else if (!bus_service_owner_in_queue (service, connection))
     {
       *result = DBUS_RELEASE_NAME_REPLY_NOT_OWNER;
     }
@@ -666,7 +680,7 @@ bus_registry_release_service (BusRegistry      *registry,
                                      transaction, error))
         goto out;
 
-      _dbus_assert (!bus_service_has_owner (service, connection));
+      _dbus_assert (!bus_service_owner_in_queue (service, connection));
       *result = DBUS_RELEASE_NAME_REPLY_RELEASED;
     }
 
@@ -1260,8 +1274,8 @@ bus_service_get_allow_replacement (BusService *service)
 }
 
 dbus_bool_t
-bus_service_has_owner (BusService     *service,
-		       DBusConnection *connection)
+bus_service_owner_in_queue (BusService     *service,
+                            DBusConnection *connection)
 {
   DBusList *link;
 
@@ -1275,8 +1289,7 @@ bus_service_has_owner (BusService     *service,
 
 dbus_bool_t 
 bus_service_list_queued_owners (BusService *service,
-                                DBusList  **return_list,
-                                DBusError  *error)
+                                DBusList  **return_list)
 {
   DBusList *link;
 
@@ -1303,6 +1316,5 @@ bus_service_list_queued_owners (BusService *service,
   
  oom:
   _dbus_list_clear (return_list);
-  BUS_SET_OOM (error);
   return FALSE;
 }
