@@ -22,6 +22,7 @@
  *
  */
 
+#include <config.h>
 #include <string.h>
 #include "dbus-internals.h"
 #include "dbus-list.h"
@@ -149,7 +150,7 @@ unquote_string_inplace (char* str, char** end)
  * through literally instead of being expanded). This function is
  * guaranteed to succeed if applied to the result of
  * _dbus_shell_quote(). If it fails, it returns %NULL.
- * The @quoted_string need not actually contain quoted or
+ * The @p quoted_string need not actually contain quoted or
  * escaped text; _dbus_shell_unquote() simply goes through the string and
  * unquotes/unescapes anything that the shell would. Both single and
  * double quotes are handled, as are escapes including escaped
@@ -162,7 +163,7 @@ unquote_string_inplace (char* str, char** end)
  * be escaped with backslash. Otherwise double quotes preserve things
  * literally.
  *
- * @quoted_string: shell-quoted string
+ * @param quoted_string shell-quoted string
  **/
 char*
 _dbus_shell_unquote (const char *quoted_string)
@@ -418,15 +419,15 @@ tokenize_command_line (const char *command_line, DBusError *error)
             {
             case '\n':
               if (!delimit_token (&current_token, &retval, error))
-		goto error;
+                goto error;
 
-		_dbus_string_free (&current_token);
+              _dbus_string_free (&current_token);
 
-		if (!_dbus_string_init (&current_token))
-		  {
-		    _DBUS_SET_OOM (error);
-		    goto init_error;
-		  }
+              if (!_dbus_string_init (&current_token))
+                {
+                  _DBUS_SET_OOM (error);
+                  goto init_error;
+                }
 
               break;
 
@@ -522,12 +523,7 @@ tokenize_command_line (const char *command_line, DBusError *error)
   _dbus_string_free (&current_token);
 
  init_error:
-  if (retval)
-    {
-      _dbus_list_foreach (&retval, (DBusForeachFunction) dbus_free, NULL);
-      _dbus_list_clear (&retval);
-    }
-
+  _dbus_list_clear_full (&retval, dbus_free);
   return NULL;
 }
 
@@ -543,10 +539,10 @@ tokenize_command_line (const char *command_line, DBusError *error)
  * does contain such expansions, they are passed through
  * literally. Free the returned vector with dbus_free_string_array().
  * 
- * @command_line: command line to parse
- * @argcp: return location for number of args
- * @argvp: return location for array of args
- * @error: error information
+ * @param command_line command line to parse
+ * @param argcp return location for number of args
+ * @param argvp return location for array of args
+ * @param error error information
  **/
 dbus_bool_t
 _dbus_shell_parse_argv (const char *command_line,
@@ -617,9 +613,8 @@ _dbus_shell_parse_argv (const char *command_line,
       ++i;
     }
   argv[argc] = NULL;
-  
-  _dbus_list_foreach (&tokens, (DBusForeachFunction) dbus_free, NULL);
-  _dbus_list_clear (&tokens);
+
+  _dbus_list_clear_full (&tokens, dbus_free);
   
   if (argcp)
     *argcp = argc;
@@ -632,8 +627,7 @@ _dbus_shell_parse_argv (const char *command_line,
   return TRUE;
 
  error:
-  _dbus_list_foreach (&tokens, (DBusForeachFunction) dbus_free, NULL);
-  _dbus_list_clear (&tokens);
+  _dbus_list_clear_full (&tokens, dbus_free);
 
   return FALSE;
 
