@@ -26,11 +26,14 @@
 
 #include <dbus/dbus-types.h>
 
+dbus_bool_t _dbus_address_test           (const char *test_data_dir);
 dbus_bool_t _dbus_auth_test              (const char *test_data_dir);
 dbus_bool_t _dbus_credentials_test       (const char *test_data_dir);
 dbus_bool_t _dbus_marshal_byteswap_test  (const char *test_data_dir);
 dbus_bool_t _dbus_marshal_validate_test  (const char *test_data_dir);
+dbus_bool_t _dbus_mem_pool_test          (const char *test_data_dir);
 dbus_bool_t _dbus_string_test            (const char *test_data_dir);
 dbus_bool_t _dbus_sysdeps_test           (const char *test_data_dir);
+dbus_bool_t _dbus_sha_test               (const char *test_data_dir);
 
 #endif
